/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.example.struts.admin.actions;



import java.security.Permission;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jguard.core.authorization.permissions.PermissionUtils;
import net.sf.jguard.example.struts.actions.BaseAction;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authorization.AuthorizationException;
import net.sf.jguard.ext.authorization.manager.AuthorizationManager;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
/**
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 *
 */
public class PermissionDispatchAction extends BaseAction{
	private static Logger logger = Logger.getLogger(PermissionDispatchAction.class);

    /**
     * create a new URLPermission.
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward create(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager am = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);

        Permission permission = null;
        try {
        	permission = getPermission(dyna);
            am.createPermission(permission,(String)dyna.get("domainName"));
        } catch (AuthorizationException e) {
            logger.error(" permission "+permission.getName()+ " in domain "+(String)dyna.get("domainName")+" not created ",e);
        } catch (ClassNotFoundException e) {
        	  logger.error(" permission "+permission.getName()+ " in domain "+(String)dyna.get("domainName")+" not created ",e);
		}

        return mapping.findForward("createPermissionOK");

    }


    /**
     * delete permission.
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward delete(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager am = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        try {
            am.deletePermission((String)dyna.get("permissionName"));
        } catch (AuthorizationException e) {
        	  logger.error(" permission "+(String)dyna.get("permissionName")+" not deleted ",e);
        }

        return mapping.findForward("deletePermissionOK");

    }

    /**
     * update permission.
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward update(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager am = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        Permission permission = null;

        try {
        	permission = getPermission(dyna);
            am.updatePermission((String)dyna.get("oldPermissionName"),permission,(String)dyna.get("domainName"));
        } catch (AuthorizationException e) {
        	logger.error(" permission "+(String)dyna.get("oldPermissionName")+" not updated ",e);
        } catch (ClassNotFoundException e) {
        	logger.error(" permission "+(String)dyna.get("oldPermissionName")+" not updated ",e);
		}

        return mapping.findForward("updatePermissionOK");

    }


	private Permission getPermission(DynaActionForm dyna) throws ClassNotFoundException {
		String permissionName = (String)dyna.get("permissionName");
        String permissionClassName = (String)dyna.get("permissionClass");
        String actions =(String)dyna.get("permissionActions");
        Permission permission = PermissionUtils.getPermission(permissionClassName,permissionName,actions);
		return permission;
	}
}
