/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.example.struts.admin.actions;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jguard.core.CoreConstants;
import net.sf.jguard.core.authentication.AuthenticationException;
import net.sf.jguard.core.authentication.manager.AuthenticationManager;
import net.sf.jguard.core.authorization.permissions.Domain;
import net.sf.jguard.core.authorization.permissions.JGPermissionCollection;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.example.struts.actions.BaseAction;
import net.sf.jguard.ext.SecurityConstants;
import net.sf.jguard.ext.authorization.AuthorizationException;
import net.sf.jguard.ext.authorization.manager.AuthorizationManager;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;

import salvo.jesus.graph.DirectedGraphImpl;
import salvo.jesus.graph.Graph;
import salvo.jesus.graph.Vertex;
import salvo.jesus.graph.VertexImpl;
import salvo.jesus.graph.visual.VisualGraph;
import salvo.jesus.graph.visual.layout.GraphLayoutManager;
import salvo.jesus.graph.visual.layout.StraightLineLayout;


/**
 * @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
 */
public class PrincipalDispatchAction extends BaseAction{

	private static Logger logger = Logger.getLogger(PrincipalDispatchAction.class);

    /**
     * list principals (Principals).
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward list(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        DynaActionForm dyna = (DynaActionForm)form;
        Set roles = null;

        AuthorizationManager am = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        roles = am.listPrincipals();
        Iterator itPrincipals = roles.iterator();
        List principalsList = new ArrayList();
        while(itPrincipals.hasNext()){
            principalsList.add(itPrincipals.next());
        }
        dyna.set("principals",principalsList);
        return mapping.findForward("listPrincipalsOK");

    }

    /**
     * Add inheritance beteween two existings principals.
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward deleteInheritance(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager am = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);

        try {
            am.deleteInheritance((String)dyna.get("principalAsc"), (String)dyna.get("principalDesc"));
        } catch (AuthorizationException e) {
            e.printStackTrace();
        }

        return mapping.findForward("addInheritanceOK");
    }

    /**
     * Delete inheritance beteween two existings principals.
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward addInheritance(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager am = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);

        try {
            am.addInheritance((String)dyna.get("principalAsc"), (String)dyna.get("principalDesc"));
        } catch (AuthorizationException e) {
            e.printStackTrace();
        }

        return mapping.findForward("addInheritanceOK");
    }


    /**
     * create a new role(Principal).
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward create(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager authorizationManager = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        AuthenticationManager authenticationManager = (AuthenticationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHENTICATION_MANAGER);

        try {
        	RolePrincipal principal = new RolePrincipal((String)dyna.get("principalName"),request.getSession().getServletContext().getServletContextName());
            authorizationManager.createPrincipal(principal);
            authenticationManager.createPrincipal(principal);
        } catch (AuthorizationException e) {
            logger.error(e.getMessage());
        } catch (AuthenticationException e) {
        	logger.error(e.getMessage());
		}

        return mapping.findForward("createPrincipalOK");

    }

    /**
     * create a new role(Principal).
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward update(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager authorizationManager = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        AuthenticationManager authenticationManager = (AuthenticationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHENTICATION_MANAGER);
        RolePrincipal principal = new RolePrincipal((String)dyna.get("principalName"),(String)request.getSession().getServletContext().getAttribute(CoreConstants.APPLICATION_NAME));
        String domainNames = (String)dyna.get("domainNames");
        String permissionNames = (String)dyna.get("permissionNames");
        String[] domNames = domainNames.split("#");
        String[] permNames = permissionNames.split("#");
        if(!domainNames.equals("")){
            Set doms = authorizationManager.getDomains(Arrays.asList(domNames));
            principal.setDomains(doms);
        }
        if(!permissionNames.equals("")){
            Set perms = authorizationManager.getPermissions(Arrays.asList(permNames));
            principal.setPermissions(perms);
        }

        try {
            authorizationManager.updatePrincipal((String)dyna.get("oldPrincipalName"),principal);
            authenticationManager.updatePrincipal((String)dyna.get("oldPrincipalName"),principal);
        } catch (AuthorizationException e) {
            e.printStackTrace();
	    } catch (AuthenticationException e) {
	        e.printStackTrace();
        }

        return mapping.findForward("updatePrincipalOK");

    }

    /**
     * create a new role(Principal).
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward delete(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager authorizationManager = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        AuthenticationManager authenticationManager = (AuthenticationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHENTICATION_MANAGER);
        try {
            authorizationManager.deletePrincipal((new RolePrincipal((String)dyna.get("principalName"))));
            authenticationManager.deletePrincipal((new RolePrincipal((String)dyna.get("principalName"))));
        } catch (AuthorizationException e) {
            e.printStackTrace();
        } catch (AuthenticationException e) {
            e.printStackTrace();
        }

        return mapping.findForward("deletePrincipalOK");

    }

    /**
     * read a role(Principal).
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward read(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        AuthorizationManager authorizationManager = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        RolePrincipal principal = null;
        Set domains = null;
        Set domainsNotBound = null;
        try {
            principal = (RolePrincipal) authorizationManager.readPrincipal((String)dyna.get("principalName"));
            domains = authorizationManager.listDomains();
            domainsNotBound = new HashSet(domains);
        } catch (AuthorizationException e) {
            logger.error("principal "+(String)dyna.get("principalName")+" cannot be read",e);
        }
        domainsNotBound.removeAll(principal.getDomains());
        List domainsNBList = new ArrayList(domainsNotBound);
        dyna.set("DomainsNotBound",domainsNBList);

        dyna.set("principal",principal);
        List domainsList = new ArrayList(domains);
        dyna.set("Domains",domainsList);

        //permissions not owned by role directly
        //and not owned by a domain owned by a role
        Set permissionsNotBound = new HashSet();
        Iterator itDomainsNB = domainsNotBound.iterator();
        while(itDomainsNB.hasNext()){
            Domain domainTmp = (Domain)itDomainsNB.next();
            try {
                permissionsNotBound.add(domainTmp.clone());
            } catch (CloneNotSupportedException e1) {
                logger.error(" domain "+domainTmp+" cannot be cloned",e1);
            }
        }
        Iterator itPermNB = permissionsNotBound.iterator();
       while(itPermNB.hasNext()){
           JGPermissionCollection domain = (JGPermissionCollection)itPermNB.next();
           domain.getPermissions().removeAll(principal.getOrphanedPermissions());
       }
       List permissionsNB = new ArrayList(permissionsNotBound);

       dyna.set("permissionsNotBound",permissionsNB);
       return mapping.findForward("readPrincipalOK");

    }

    /**
     * clone a role(Principal).
     * @param mapping
     * @param form
     * @param request
     * @param response
     * @return
     */
    public ActionForward clone(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {

        DynaActionForm dyna = (DynaActionForm)form;
        String principalToClone = (String)dyna.get("principalName");
        AuthorizationManager authorizationManager = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
        AuthenticationManager authenticationManager = (AuthenticationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHENTICATION_MANAGER);
        try {
        	Principal clonedPpal = authorizationManager.clonePrincipal(principalToClone);
            authenticationManager.createPrincipal(clonedPpal);
        } catch (AuthorizationException e) {
            logger.error(e.getMessage());
        } catch (AuthenticationException e) {
        	logger.error(e.getMessage());
		}
        return mapping.findForward("clonePrincipalOK");
    }

    public ActionForward displayPrincipalGraph(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) {
        //load the role hierarchy into a Graph
        Map vertices = new HashMap();
        Graph graph = new DirectedGraphImpl();

        try {
            AuthorizationManager authorizationManager = (AuthorizationManager) request.getSession().getServletContext().getAttribute(SecurityConstants.AUTHORIZATION_MANAGER);
            Set principals = authorizationManager.listPrincipals();

            //create all vertex and put into map
            for (Iterator it = principals.iterator(); it.hasNext();) {
                RolePrincipal principal = (RolePrincipal) it.next();
                String principalName = principal.getLocalName();

                Vertex vertex = new VertexImpl(principalName);
                graph.add(vertex);
                vertices.put(principalName, vertex);
            }

            //reiterate all principals to generate the links between principals
            for (Iterator it = principals.iterator(); it.hasNext();) {
                RolePrincipal principal = (RolePrincipal) it.next();
                String principalName = principal.getLocalName();
                Vertex ascendantVertex = (Vertex) vertices.get(principalName);

                for (Iterator it2 = principal.getDescendants().iterator(); it2.hasNext();) {
                    RolePrincipal principalDesc = (RolePrincipal) it2.next();
                    String principalDescName = principalDesc.getLocalName();
                    Vertex descendantVertex = (Vertex) vertices.get(principalDescName);

                    graph.addEdge(ascendantVertex, descendantVertex);
                }
            }

            //paint the graph
            VisualGraph visualGraph = new VisualGraph();
            visualGraph.setGraph(graph);

            GraphLayoutManager layoutManager = new StraightLineLayout(visualGraph);
            visualGraph.setGraphLayoutManager(layoutManager);
            visualGraph.layout();

            int width = visualGraph.getMaxSize().width + 20;
            int height =  visualGraph.getMaxSize().height + 20;

            BufferedImage bufferedImage = new BufferedImage(width, height ,BufferedImage.TYPE_INT_RGB);
            Graphics2D graphics2D = bufferedImage.createGraphics();
            graphics2D.fillRect(0, 0, width, height);
            visualGraph.paint(graphics2D);
            graphics2D.dispose();

            String resize = request.getParameter("resize");

            response.setContentType("image/png");

            if (Boolean.valueOf(resize).booleanValue())
                ImageIO.write(resize(bufferedImage, 640), "png", response.getOutputStream());
            else
                ImageIO.write(bufferedImage, "png", response.getOutputStream());
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

	/**
	 *
	 * @param src
	 * @param width
	 * @return
	 */
	private static BufferedImage resize(BufferedImage src, int width) {
	    double d = src.getWidth();
	    double d1 = src.getHeight();
	    double d2 = width;
	    double d3 = d2 / d;

	    if (d1 * d3 > d2) {
	        d3 = d2 / d1;
	    }

	    if (d3 > 0.8D) {
	        d3 = 1.0D;
	    }

	    int j = (int) (d * d3);
	    int k = (int) (d1 * d3);

	    AffineTransform tx = new AffineTransform();
	    tx.scale(d3, d3);

	    RenderingHints rh = new RenderingHints(RenderingHints.KEY_ANTIALIASING,
	            RenderingHints.VALUE_ANTIALIAS_ON);
	    rh.put(RenderingHints.KEY_ALPHA_INTERPOLATION,
	            RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
	    rh.put(RenderingHints.KEY_COLOR_RENDERING,
	            RenderingHints.VALUE_COLOR_RENDER_QUALITY);
	    rh.put(RenderingHints.KEY_INTERPOLATION,
	            RenderingHints.VALUE_INTERPOLATION_BICUBIC);
	    rh.put(RenderingHints.KEY_RENDERING,
	            RenderingHints.VALUE_RENDER_QUALITY);
	    rh.put(RenderingHints.KEY_STROKE_CONTROL,
	            RenderingHints.VALUE_STROKE_PURE);
	    rh.put(RenderingHints.KEY_TEXT_ANTIALIASING,
	            RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

	    AffineTransformOp op = new AffineTransformOp(tx, rh);
	    BufferedImage biNew = new BufferedImage(j, k, src.getType());

	    op.filter(src, biNew);

	    return biNew;
	}
	
}
